@@ -3,3 +3,6 @@ pub mod classic;
 
 /// Famous social networks.
 pub mod social;
+
+/// Random graphs
+pub mod random;
\ No newline at end of file
