@@ -49,7 +49,7 @@ let graph = Graph::<&str, ()>::new_from_nodes_and_edges(
 ```
 */
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-pub struct Graph<T: PartialOrd, A: Copy> {
+pub struct Graph<T: PartialOrd + Send, A: Copy> {
     /// The graph's nodes, stored as a `HashMap` keyed by the node names.
     nodes: HashMap<T, Node<T, A>>,
     /// The graph's edges, stored as a `HashMap` keyed by a tuple of node names.
@@ -66,7 +66,7 @@ pub struct Graph<T: PartialOrd, A: Copy> {
     predecessors: HashMap<T, HashSet<T>>,
 }
 
-impl<T: Display + PartialOrd, A: Copy> Graph<T, A> {
+impl<T: Display + PartialOrd + Send + Sync, A: Copy> Graph<T, A> {
     /**
     Adds an `edge` to the `Graph`.
 
@@ -76,9 +76,9 @@ impl<T: Display + PartialOrd, A: Copy> Graph<T, A> {
     ```
     use graphrs::{Edge, Graph, GraphSpecs};
 
-    let mut graph: Graph<&str, ()> = Graph::new(GraphSpecs::directed());
+    let mut graph: Graph<&str, ()> = Graph::new(GraphSpecs::directed_create_missing());
     let result = graph.add_edge(Edge::new("n1", "n2"));
-    assert!(result.is_err());
+    assert!(result.is_ok());
     ```
     */
     pub fn add_edge(&mut self, edge: Edge<T, A>) -> Result<(), Error>
@@ -176,6 +176,27 @@ impl<T: Display + PartialOrd, A: Copy> Graph<T, A> {
         Ok(())
     }
 
+    /**
+    Adds an edge, as a (u, v) tuple, to the `Graph`.
+
+    If the new edge references nodes that don't exist the graph's `specs.missing_node_strategy`
+    determines what happens.
+
+    ```
+    use graphrs::{Edge, Graph, GraphSpecs};
+
+    let mut graph: Graph<&str, ()> = Graph::new(GraphSpecs::directed_create_missing());
+    let result = graph.add_edge_tuple("n1", "n2");
+    assert!(result.is_ok());
+    ```
+    */
+    pub fn add_edge_tuple(&mut self, u: T, v: T) -> Result<(), Error>
+    where
+        T: Hash + Eq + Copy + Ord + Display
+    {
+        self.add_edge(Edge::new(u, v))
+    }
+
     /**
     Adds new edges to a `Graph`, or updates existing edges, or both.
 
@@ -212,6 +233,42 @@ impl<T: Display + PartialOrd, A: Copy> Graph<T, A> {
         Ok(())
     }
 
+    /**
+    Adds new edges to a `Graph`, or updates existing edges, or both.
+
+    If the new edges reference nodes that don't exist the graph's `specs.missing_node_strategy`
+    determines what happens.
+
+    # Arguments
+
+    * `edges`: the new edges to add to the graph
+
+    ```
+    use graphrs::{Edge, Graph, GraphSpecs};
+
+    let mut graph: Graph<&str, ()> = Graph::new(GraphSpecs::directed_create_missing());
+    let result = graph.add_edge_tuples(vec![
+        ("n1", "n2"),
+        ("n2", "n3"),
+    ]);
+    assert!(result.is_ok());
+    ```
+    */
+    pub fn add_edge_tuples(&mut self, edges: Vec<(T, T)>) -> Result<(), Error>
+    where
+        T: Hash + Eq + Copy + Ord + Display,
+        A: Copy,
+    {
+        for edge in edges {
+            let result = self.add_edge(Edge::new(edge.0, edge.1));
+            if result.is_err() {
+                return result;
+            }
+        }
+
+        Ok(())
+    }
+
     /**
     Adds a node to the graph or updates the node's attributes if the node already exists.
 
